#include <iostream>
#include "../../IsoSpec++/unity-build.cpp"

using std::cout;
using std::endl;

using namespace IsoSpec;

int main(void){
    // int isotopeNumbers[] = {2, 3};
    // int config_size = isotopeNumbers[0] + isotopeNumbers[1];
    // int atomCounts[] = {10, 10};
    // double isotopeMasses[] = {1.0, 2.0, 3.0, 4.0, 5.0};
    // double isotopeProbabilities[] = {0.5, 0.5, 0.5, 0.3, 0.2};

    // IsoThresholdGenerator* generator = new IsoThresholdGenerator(
    //     2,
    //     isotopeNumbers,
    //     atomCounts,
    //     isotopeMasses,
    //     isotopeProbabilities,
    //     .001,
    //     true,
    //     1000,
    //     1000);

    // cout << reinterpret_cast<IsoThresholdGenerator*>(generator)->getAllDim() << endl;
    // void* tabulator = setupThresholdTabulator(generator, true, true, true, true);

    return 0;
}
